@@ -26,6 +26,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "JackMutex.h"
 #include "JackTransportEngine.h"
 #include "JackPlatformPlug.h"
+#include "JackRequest.h"
+#include "JackChannel.h"
 #include <map>
 
 namespace Jack
@@ -47,22 +49,23 @@ class SERVER_EXPORT JackEngine : public JackLockAble
 
         JackGraphManager* fGraphManager;
         JackEngineControl* fEngineControl;
+        char fSelfConnectMode;
         JackClientInterface* fClientTable[CLIENT_NUM];
         JackSynchro* fSynchroTable;
         JackServerNotifyChannel fChannel;              /*! To communicate between the RT thread and server */
         JackProcessSync fSignal;
         jack_time_t fLastSwitchUsecs;
+        JackMetadata fMetadata;
 
         int fSessionPendingReplies;
-        JackChannelTransaction *fSessionTransaction;
-        JackSessionNotifyResult *fSessionResult;
+        detail::JackChannelTransactionInterface* fSessionTransaction;
+        JackSessionNotifyResult* fSessionResult;
         std::map<int,std::string> fReservationMap;
-        int fMaxUUID;
 
-        int ClientCloseAux(int refnum, JackClientInterface* client, bool wait);
+        int ClientCloseAux(int refnum, bool wait);
         void CheckXRun(jack_time_t callback_usecs);
 
-        int NotifyAddClient(JackClientInterface* new_client, const char* name, int refnum);
+        int NotifyAddClient(JackClientInterface* new_client, const char* new_name, int refnum);
         void NotifyRemoveClient(const char* name, int refnum);
 
         void ProcessNext(jack_time_t callback_usecs);
@@ -72,7 +75,9 @@ class SERVER_EXPORT JackEngine : public JackLockAble
         bool GenerateUniqueName(char* name);
 
         int AllocateRefnum();
-        void ReleaseRefnum(int ref);
+        void ReleaseRefnum(int refnum);
+
+        int ClientNotify(JackClientInterface* client, int refnum, const char* name, int notify, int sync, const char* message, int value1, int value2);
 
         void NotifyClient(int refnum, int event, int sync, const char*  message, int value1, int value2);
         void NotifyClients(int event, int sync, const char*  message,  int value1, int value2);
@@ -82,25 +87,27 @@ class SERVER_EXPORT JackEngine : public JackLockAble
         void NotifyPortRename(jack_port_id_t src, const char* old_name);
         void NotifyActivate(int refnum);
 
-        int GetNewUUID();
-        void EnsureUUID(int uuid);
+        void EnsureUUID(jack_uuid_t uuid);
 
         bool CheckClient(int refnum)
         {
             return (refnum >= 0 && refnum < CLIENT_NUM && fClientTable[refnum] != NULL);
         }
 
+        int CheckPortsConnect(int refnum, jack_port_id_t src, jack_port_id_t dst);
+
     public:
 
-        JackEngine(JackGraphManager* manager, JackSynchro* table, JackEngineControl* controler);
+        JackEngine(JackGraphManager* manager, JackSynchro* table, JackEngineControl* controler, char self_connect_mode);
         ~JackEngine();
 
         int Open();
         int Close();
 
         // Client management
-        int ClientCheck(const char* name, int uuid, char* name_res, int protocol, int options, int* status);
-        int ClientExternalOpen(const char* name, int pid, int uuid, int* ref, int* shared_engine, int* shared_client, int* shared_graph_manager);
+        int ClientCheck(const char* name, jack_uuid_t uuid, char* name_res, int protocol, int options, int* status);
+
+        int ClientExternalOpen(const char* name, int pid, jack_uuid_t uuid, int* ref, int* shared_engine, int* shared_client, int* shared_graph_manager);
         int ClientInternalOpen(const char* name, int* ref, JackEngineControl** shared_engine, JackGraphManager** shared_manager, JackClientInterface* client, bool wait);
 
         int ClientExternalClose(int refnum);
@@ -109,6 +116,8 @@ class SERVER_EXPORT JackEngine : public JackLockAble
         int ClientActivate(int refnum, bool is_real_time);
         int ClientDeactivate(int refnum);
 
+        void ClientKill(int refnum);
+
         int GetClientPID(const char* name);
         int GetClientRefNum(const char* name);
 
@@ -129,15 +138,19 @@ class SERVER_EXPORT JackEngine : public JackLockAble
 
         int PortRename(int refnum, jack_port_id_t port, const char* name);
 
+        int PortSetDefaultMetadata(jack_port_id_t port, const char* pretty_name);
+
         int ComputeTotalLatencies();
 
+        int PropertyChangeNotify(jack_uuid_t subject, const char* key,jack_property_change_t change);
+
         // Graph
         bool Process(jack_time_t cur_cycle_begin, jack_time_t prev_cycle_end);
 
         // Notifications
-        void NotifyXRun(jack_time_t callback_usecs, float delayed_usecs);
+        void NotifyDriverXRun();
+        void NotifyClientXRun(int refnum);
         void NotifyFailure(int code, const char* reason);
-        void NotifyXRun(int refnum);
         void NotifyGraphReorder();
         void NotifyBufferSize(jack_nframes_t buffer_size);
         void NotifySampleRate(jack_nframes_t sample_rate);
@@ -145,13 +158,13 @@ class SERVER_EXPORT JackEngine : public JackLockAble
         void NotifyQuit();
 
         // Session management
-        void SessionNotify(int refnum, const char *target, jack_session_event_type_t type, const char *path, JackChannelTransaction *socket);
-        void SessionReply(int refnum);
+        void SessionNotify(int refnum, const char *target, jack_session_event_type_t type, const char *path, detail::JackChannelTransactionInterface *socket, JackSessionNotifyResult** result);
+        int SessionReply(int refnum);
 
-        void GetUUIDForClientName(const char *client_name, char *uuid_res, int *result);
-        void GetClientNameForUUID(const char *uuid, char *name_res, int *result);
-        void ReserveClientName(const char *name, const char *uuid, int *result);
-        void ClientHasSessionCallbackRequest(const char *name, int *result);
+        int GetUUIDForClientName(const char *client_name, char *uuid_res);
+        int GetClientNameForUUID(const char *uuid, char *name_res);
+        int ReserveClientName(const char *name, const char *uuid);
+        int ClientHasSessionCallback(const char *name);
 };
 
 
