@@ -36,12 +36,12 @@ JackExternalClient::~JackExternalClient()
 int JackExternalClient::ClientNotify(int refnum, const char* name, int notify, int sync, const char* message, int value1, int value2)
 {
     int result = -1;
-    jack_log("JackExternalClient::ClientNotify ref = %ld name = %s notify = %ld", refnum, name, notify);
+    jack_log("JackExternalClient::ClientNotify ref = %ld client = %s name = %s notify = %ld", refnum, fClientControl->fName, name, notify);
     fChannel.ClientNotify(refnum, name, notify, sync, message, value1, value2, &result);
     return result;
 }
 
-int JackExternalClient::Open(const char* name, int pid, int refnum, int uuid, int* shared_client)
+int JackExternalClient::Open(const char* name, int pid, int refnum, jack_uuid_t uuid, int* shared_client)
 {
     try {
 
@@ -49,28 +49,29 @@ int JackExternalClient::Open(const char* name, int pid, int refnum, int uuid, in
             jack_error("Cannot connect to client name = %s\n", name);
             return -1;
         }
-        
+
         // Use "placement new" to allocate object in shared memory
         JackShmMemAble* shared_mem = static_cast<JackShmMemAble*>(JackShmMem::operator new(sizeof(JackClientControl)));
         shared_mem->Init();
         fClientControl = new(shared_mem) JackClientControl(name, pid, refnum, uuid);
-      
+
         if (!fClientControl) {
             jack_error("Cannot allocate client shared memory segment");
             return -1;
         }
-       
+
         *shared_client = shared_mem->GetShmIndex();
         jack_log("JackExternalClient::Open name = %s index = %ld base = %x", name, shared_mem->GetShmIndex(), shared_mem->GetShmAddress());
         return 0;
 
-    } catch (std::exception e) {
+    } catch (std::exception&) {
         return -1;
     }
 }
 
 int JackExternalClient::Close()
 {
+    jack_log("JackExternalClient::Close");
     fChannel.Close();
     if (fClientControl) {
         fClientControl->~JackClientControl();
