@@ -31,12 +31,12 @@ namespace Jack
 
 JackServer* JackServerGlobals::fInstance;
 unsigned int JackServerGlobals::fUserCount;
-int JackServerGlobals::fRTNotificationSocket;
 std::map<std::string, JackDriverInfo*> JackServerGlobals::fSlavesList;
 std::map<std::string, int> JackServerGlobals::fInternalsList;
 
 bool (* JackServerGlobals::on_device_acquire)(const char * device_name) = NULL;
 void (* JackServerGlobals::on_device_release)(const char * device_name) = NULL;
+void (* JackServerGlobals::on_device_reservation_loop)(void) = NULL;
 
 int JackServerGlobals::Start(const char* server_name,
                              jack_driver_desc_t* driver_desc,
@@ -48,10 +48,11 @@ int JackServerGlobals::Start(const char* server_name,
                              int priority,
                              int port_max,
                              int verbose,
-                            jack_timer_type_t clock)
+                             jack_timer_type_t clock,
+                             char self_connect_mode)
 {
     jack_log("Jackdmp: sync = %ld timeout = %ld rt = %ld priority = %ld verbose = %ld ", sync, time_out_ms, rt, priority, verbose);
-    new JackServer(sync, temporary, time_out_ms, rt, priority, port_max, verbose, clock, server_name);  // Will setup fInstance and fUserCount globals
+    new JackServer(sync, temporary, time_out_ms, rt, priority, port_max, verbose, clock, self_connect_mode, server_name);  // Will setup fInstance and fUserCount globals
     int res = fInstance->Open(driver_desc, driver_params);
     return (res < 0) ? res : fInstance->Start();
 }
@@ -84,7 +85,7 @@ void JackServerGlobals::Delete()
         int status;
         int refnum = (*it2).second;
         if (refnum > 0) {
-            // Client object is internally kept in JackEngine, and will be desallocated in InternalClientUnload
+            // Client object is internally kept in JackEngine, and will be deallocated in InternalClientUnload
             fInstance->GetEngine()->InternalClientUnload(refnum, &status);
         }
     }
@@ -100,9 +101,7 @@ bool JackServerGlobals::Init()
     int client_timeout = 0; /* msecs; if zero, use period size. */
     int realtime_priority = 10;
     int verbose_aux = 0;
-    int do_mlock = 1;
     unsigned int port_max = 128;
-    int do_unlock = 0;
     int temporary = 0;
 
     int opt = 0;
@@ -117,7 +116,7 @@ bool JackServerGlobals::Init()
     int loopback = 0;
     int sync = 0;
     int rc, i;
-    int ret;
+    int res;
     int replace_registry = 0;
 
     FILE* fp = 0;
@@ -170,18 +169,18 @@ bool JackServerGlobals::Init()
         if (!fp) {
             fp = fopen("/etc/jackdrc", "r");
         }
-        // if still not found, check old config name for backwards compatability
+        // if still not found, check old config name for backwards compatibility
         if (!fp) {
             fp = fopen("/etc/jackd.conf", "r");
         }
 
         argc = 0;
         if (fp) {
-            ret = fscanf(fp, "%s", buffer);
-            while (ret != 0 && ret != EOF) {
+            res = fscanf(fp, "%s", buffer);
+            while (res != 0 && res != EOF) {
                 argv[argc] = (char*)malloc(64);
                 strcpy(argv[argc], buffer);
-                ret = fscanf(fp, "%s", buffer);
+                res = fscanf(fp, "%s", buffer);
                 argc++;
             }
             fclose(fp);
@@ -205,7 +204,11 @@ bool JackServerGlobals::Init()
                     if (tolower (optarg[0]) == 'h') {
                         clock_source = JACK_TIMER_HPET;
                     } else if (tolower (optarg[0]) == 'c') {
-                        clock_source = JACK_TIMER_CYCLE_COUNTER;
+                        /* For backwards compatibility with scripts, allow
+                         * the user to request the cycle clock on the
+                         * command line, but use the system clock instead
+                         */
+                        clock_source = JACK_TIMER_SYSTEM_CLOCK;
                     } else if (tolower (optarg[0]) == 's') {
                         clock_source = JACK_TIMER_SYSTEM_CLOCK;
                     } else {
@@ -234,11 +237,9 @@ bool JackServerGlobals::Init()
                     break;
 
                 case 'm':
-                    do_mlock = 0;
                     break;
 
                 case 'u':
-                    do_unlock = 1;
                     break;
 
                 case 'v':
@@ -315,8 +316,9 @@ bool JackServerGlobals::Init()
         }
 
 #ifndef WIN32
-        if (server_name == NULL)
+        if (server_name == NULL) {
             server_name = (char*)JackTools::DefaultServerName();
+        }
 #endif
 
         rc = jack_register_server(server_name, false);
@@ -338,14 +340,15 @@ bool JackServerGlobals::Init()
         jack_cleanup_shm();
         JackTools::CleanupFiles(server_name);
 
-        if (!realtime && client_timeout == 0)
+        if (!realtime && client_timeout == 0) {
             client_timeout = 500; /* 0.5 sec; usable when non realtime. */
+        }
 
         for (i = 0; i < argc; i++) {
             free(argv[i]);
         }
 
-        int res = Start(server_name, driver_desc, master_driver_params, sync, temporary, client_timeout, realtime, realtime_priority, port_max, verbose_aux, clock_source);
+        int res = Start(server_name, driver_desc, master_driver_params, sync, temporary, client_timeout, realtime, realtime_priority, port_max, verbose_aux, clock_source, JACK_DEFAULT_SELF_CONNECT_MODE);
         if (res < 0) {
             jack_error("Cannot start server... exit");
             Delete();
@@ -387,14 +390,16 @@ bool JackServerGlobals::Init()
         }
     }
 
-    if (master_driver_params)
+    if (master_driver_params) {
         jack_free_driver_params(master_driver_params);
+    }
     return true;
 
 error:
     jack_log("JackServerGlobals Init error");
-    if (master_driver_params)
+    if (master_driver_params) {
         jack_free_driver_params(master_driver_params);
+    }
     Destroy();
     return false;
 }
