@@ -1,20 +1,20 @@
 /*
  Copyright (C) 2004-2008 Grame
- 
+
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.
- 
+
  You should have received a copy of the GNU Lesser General Public License
- along with this program; if not, write to the Free Software 
+ along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- 
+
  */
 
 #ifndef __JackShmMem__
@@ -33,50 +33,11 @@
 namespace Jack
 {
 
-SERVER_EXPORT void LockMemoryImp(void* ptr, size_t size);
-SERVER_EXPORT void InitLockMemoryImp(void* ptr, size_t size);
-SERVER_EXPORT void UnlockMemoryImp(void* ptr, size_t size);
-SERVER_EXPORT void LockAllMemory();
-SERVER_EXPORT void UnlockAllMemory();
-
-class JackMem
-{
-    private:
-
-        size_t fSize;
-        static size_t gSize;
-
-    protected:
-    
-        JackMem(): fSize(gSize)
-        {}
-        ~JackMem()
-        {}
-        
-    public:
-
-        void* operator new(size_t size)
-        {
-            gSize = size;
-            return calloc(1, size);
-        }
-
-        void operator delete(void* ptr, size_t size)
-        {
-            free(ptr);
-        }
-	
-        void LockMemory()
-        {
-            LockMemoryImp(this, fSize);
-        }
-
-        void UnlockMemory()
-        {
-            UnlockMemoryImp(this, fSize);
-        }
-
-};
+void LockMemoryImp(void* ptr, size_t size);
+void InitLockMemoryImp(void* ptr, size_t size);
+void UnlockMemoryImp(void* ptr, size_t size);
+void LockAllMemory();
+void UnlockAllMemory();
 
 /*!
 \brief
@@ -87,11 +48,11 @@ A class which objects possibly want to be allocated in shared memory derives fro
 class JackShmMemAble
 {
     protected:
-        
+
         jack_shm_info_t fInfo;
-        
+
     public:
-             
+
         void Init();
 
         int GetShmIndex()
@@ -126,18 +87,18 @@ class SERVER_EXPORT JackShmMem : public JackShmMemAble
 {
 
      protected:
-    
+
         JackShmMem();
         ~JackShmMem();
-  
+
     public:
-    
+
         void* operator new(size_t size);
         void* operator new(size_t size, void* memory);
-        
+
         void operator delete(void* p, size_t size);
 		void operator delete(void* p);
-   
+
 };
 
 /*!
@@ -151,18 +112,21 @@ class JackShmReadWritePtr
     private:
 
         jack_shm_info_t fInfo;
+        bool fInitDone;
 
-        void Init(int index, const char* server_name = "default")
+        void Init(int index, const char* server_name = JACK_DEFAULT_SERVER_NAME)
         {
             if (fInfo.index < 0 && index >= 0) {
-                jack_log("JackShmReadWritePtr::Init %ld %ld", index, fInfo.index);
-                if (jack_initialize_shm(server_name) < 0)
-                    throw - 1;
+                jack_log("JackShmReadWritePtr::Init %ld %d", index, fInfo.index);
+                if (jack_initialize_shm(server_name) < 0) {
+                    throw std::bad_alloc();
+                }
                 fInfo.index = index;
-                if (jack_attach_shm(&fInfo)) {
-                    throw - 2;
+                if (jack_attach_lib_shm(&fInfo)) {
+                    throw std::bad_alloc();
                 }
                 GetShmAddress()->LockMemory();
+                fInitDone = true;
             }
         }
 
@@ -171,6 +135,7 @@ class JackShmReadWritePtr
         JackShmReadWritePtr()
         {
             fInfo.index = -1;
+            fInitDone = false;
             fInfo.ptr.attached_at = (char*)NULL;
         }
 
@@ -181,10 +146,14 @@ class JackShmReadWritePtr
 
         ~JackShmReadWritePtr()
         {
+            if (!fInitDone) {
+               jack_error("JackShmReadWritePtr::~JackShmReadWritePtr - Init not done for %d, skipping unlock", fInfo.index);
+               return;
+            }
             if (fInfo.index >= 0) {
-                jack_log("JackShmReadWritePtr::~JackShmReadWritePtr %ld", fInfo.index);
+                jack_log("JackShmReadWritePtr::~JackShmReadWritePtr %d", fInfo.index);
                 GetShmAddress()->UnlockMemory();
-                jack_release_shm(&fInfo);
+                jack_release_lib_shm(&fInfo);
                 fInfo.index = -1;
              }
         }
@@ -232,24 +201,27 @@ class JackShmReadWritePtr1
     private:
 
         jack_shm_info_t fInfo;
+        bool fInitDone;
 
-        void Init(int index, const char* server_name = "default")
+        void Init(int index, const char* server_name = JACK_DEFAULT_SERVER_NAME)
         {
             if (fInfo.index < 0 && index >= 0) {
-                jack_log("JackShmReadWritePtr1::Init %ld %ld", index, fInfo.index);
-                if (jack_initialize_shm(server_name) < 0)
-                    throw - 1;
-                fInfo.index = index;
-                if (jack_attach_shm(&fInfo)) {
-                    throw - 2;
+                jack_log("JackShmReadWritePtr1::Init %ld %d", index, fInfo.index);
+                if (jack_initialize_shm(server_name) < 0) {
+                    throw std::bad_alloc();
                 }
+                fInfo.index = index;
+                if (jack_attach_lib_shm(&fInfo)) {
+                    throw std::bad_alloc();
+                }
+                GetShmAddress()->LockMemory();
+                fInitDone = true;
                 /*
                 nobody else needs to access this shared memory any more, so
                 destroy it. because we have our own attachment to it, it won't
                 vanish till we exit (and release it).
                 */
                 jack_destroy_shm(&fInfo);
-                GetShmAddress()->LockMemory();
             }
         }
 
@@ -258,6 +230,7 @@ class JackShmReadWritePtr1
         JackShmReadWritePtr1()
         {
             fInfo.index = -1;
+            fInitDone = false;
             fInfo.ptr.attached_at = NULL;
         }
 
@@ -268,10 +241,14 @@ class JackShmReadWritePtr1
 
         ~JackShmReadWritePtr1()
         {
+            if (!fInitDone) {
+               jack_error("JackShmReadWritePtr1::~JackShmReadWritePtr1 - Init not done for %d, skipping unlock", fInfo.index);
+               return;
+            }
             if (fInfo.index >= 0) {
-                jack_log("JackShmReadWritePtr1::~JackShmReadWritePtr1 %ld", fInfo.index);
+                jack_log("JackShmReadWritePtr1::~JackShmReadWritePtr1 %d", fInfo.index);
                 GetShmAddress()->UnlockMemory();
-                jack_release_shm(&fInfo);
+                jack_release_lib_shm(&fInfo);
                 fInfo.index = -1;
             }
         }
@@ -319,18 +296,21 @@ class JackShmReadPtr
     private:
 
         jack_shm_info_t fInfo;
+        bool fInitDone;
 
-        void Init(int index, const char* server_name = "default")
+        void Init(int index, const char* server_name = JACK_DEFAULT_SERVER_NAME)
         {
             if (fInfo.index < 0 && index >= 0) {
-                jack_log("JackShmPtrRead::Init %ld %ld", index, fInfo.index);
-                if (jack_initialize_shm(server_name) < 0)
-                    throw - 1;
+                jack_log("JackShmPtrRead::Init %ld %d", index, fInfo.index);
+                if (jack_initialize_shm(server_name) < 0) {
+                    throw std::bad_alloc();
+                }
                 fInfo.index = index;
-                if (jack_attach_shm_read(&fInfo)) {
-                    throw - 2;
+                if (jack_attach_lib_shm_read(&fInfo)) {
+                    throw std::bad_alloc();
                 }
                 GetShmAddress()->LockMemory();
+                fInitDone = true;
             }
         }
 
@@ -339,6 +319,7 @@ class JackShmReadPtr
         JackShmReadPtr()
         {
             fInfo.index = -1;
+            fInitDone = false;
             fInfo.ptr.attached_at = NULL;
         }
 
@@ -349,10 +330,14 @@ class JackShmReadPtr
 
         ~JackShmReadPtr()
         {
+            if (!fInitDone) {
+               jack_error("JackShmReadPtr::~JackShmReadPtr - Init not done for %ld, skipping unlock", fInfo.index);
+               return;
+            }
             if (fInfo.index >= 0) {
                 jack_log("JackShmPtrRead::~JackShmPtrRead %ld", fInfo.index);
                 GetShmAddress()->UnlockMemory();
-                jack_release_shm(&fInfo);
+                jack_release_lib_shm(&fInfo);
                 fInfo.index = -1;
             }
         }
