@@ -21,7 +21,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 #ifndef __JackTime__
 #define __JackTime__
 
-#include "types.h"
 #include "JackCompilerDeps.h"
 #include "JackTypes.h"
 
@@ -31,10 +30,12 @@ extern "C"
 #endif
 
     SERVER_EXPORT void InitTime();
+    SERVER_EXPORT void EndTime();
     SERVER_EXPORT jack_time_t GetMicroSeconds(void);
     SERVER_EXPORT void JackSleep(long usec);
-    SERVER_EXPORT void SetClockSource(jack_timer_type_t source);
-    SERVER_EXPORT const char* ClockSourceName(jack_timer_type_t source);
+
+    void SetClockSource(jack_timer_type_t source);
+    const char* ClockSourceName(jack_timer_type_t source);
 
 #ifdef __cplusplus
 }
