@@ -28,6 +28,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 //#define JACK_SCHED_POLICY SCHED_RR
 #define JACK_SCHED_POLICY SCHED_FIFO
 
+#if defined(__linux__) && !defined(SCHED_RESET_ON_FORK)
+# define SCHED_RESET_ON_FORK 0x40000000
+#endif
+
 namespace Jack
 {
 
@@ -42,7 +46,7 @@ void* JackPosixThread::ThreadHandler(void* arg)
     }
 
     // Signal creation thread when started with StartSync
-    jack_log("ThreadHandler: start");
+    jack_log("JackPosixThread::ThreadHandler : start");
     obj->fStatus = kIniting;
 
     // Call Init method
@@ -59,7 +63,7 @@ void* JackPosixThread::ThreadHandler(void* arg)
         res = runnable->Execute();
     }
 
-    jack_log("ThreadHandler: exit");
+    jack_log("JackPosixThread::ThreadHandler : exit");
     pthread_exit(0);
     return 0; // never reached
 }
@@ -112,7 +116,7 @@ int JackPosixThread::StartImp(jack_native_thread_t* thread, int priority, int re
 
     if (realtime) {
 
-        jack_log("Create RT thread");
+        jack_log("JackPosixThread::StartImp : create RT thread");
 
         if ((res = pthread_attr_setinheritsched(&attributes, PTHREAD_EXPLICIT_SCHED))) {
             jack_error("Cannot request explicit scheduling for RT thread res = %d", res);
@@ -133,7 +137,10 @@ int JackPosixThread::StartImp(jack_native_thread_t* thread, int priority, int re
         }
 
     } else {
-        jack_log("Create non RT thread");
+        jack_log("JackPosixThread::StartImp : create non RT thread");
+        if ((res = pthread_attr_setinheritsched(&attributes, PTHREAD_EXPLICIT_SCHED))) {
+            jack_log("Cannot request explicit scheduling for non RT thread res = %d", res);
+        }
     }
 
     if ((res = pthread_attr_setstacksize(&attributes, THREAD_STACK))) {
@@ -232,13 +239,21 @@ int JackPosixThread::AcquireRealTimeImp(jack_native_thread_t thread, int priorit
     memset(&rtparam, 0, sizeof(rtparam));
     rtparam.sched_priority = priority;
 
-    if ((res = pthread_setschedparam(thread, JACK_SCHED_POLICY, &rtparam)) != 0) {
-        jack_error("Cannot use real-time scheduling (RR/%d)"
-                   "(%d: %s)", rtparam.sched_priority, res,
-                   strerror(res));
-        return -1;
-    }
-    return 0;
+    jack_log("JackPosixThread::AcquireRealTimeImp priority = %d", priority);
+
+    if ((res = pthread_setschedparam(thread, JACK_SCHED_POLICY, &rtparam)) == 0)
+        return 0;
+
+#ifdef SCHED_RESET_ON_FORK
+    jack_log("pthread_setschedparam() failed (%d), trying with SCHED_RESET_ON_FORK.", res);
+    if ((res = pthread_setschedparam(thread, JACK_SCHED_POLICY|SCHED_RESET_ON_FORK, &rtparam)) == 0)
+        return 0;
+#endif
+
+    jack_error("Cannot use real-time scheduling (RR/%d)"
+               " (%d: %s)", rtparam.sched_priority, res,
+               strerror(res));
+    return -1;
 }
 
 int JackPosixThread::DropRealTime()
