@@ -57,9 +57,9 @@ namespace Jack { typedef JackFifo JackSynchro; }
 #include "JackSocket.h"
 namespace Jack { typedef JackClientSocket JackChannelTransaction; }
 
-#include "JackProcessSync.h"
 /* __JackPlatformProcessSync__ */
-/* Only on windows a special JackProcessSync is used. It is directly defined by including JackProcessSync.h here */
+#include "JackPosixProcessSync.h"
+namespace Jack { typedef JackPosixProcessSync JackProcessSync; }
 
 /* __JackPlatformServerChannel__ */ 
 #include "JackSocketServerChannel.h"
