@@ -1,32 +1,42 @@
 /*
  Copyright (C) 2004-2008 Grame
- 
+
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation; either version 2.1 of the License, or
  (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.
- 
+
  You should have received a copy of the GNU Lesser General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- 
+
  */
 
 
 #ifndef __JackSystemDeps_WIN32__
 #define __JackSystemDeps_WIN32__
 
+#ifdef __MINGW32__
+#include <winsock2.h>
+#endif
 #include <windows.h>
+#include "JackCompilerDeps.h"
 
+#ifndef PATH_MAX
+#define PATH_MAX   512
+#endif
+
+#ifndef UINT32_MAX
 #define UINT32_MAX 4294967295U
+#endif
 
 #define DRIVER_HANDLE HINSTANCE
-#define LoadDriverModule(name) LoadLibrary((name))
+#define LoadDriverModule(name) LoadLibraryW((name))
 #define UnloadDriverModule(handle) (FreeLibrary(((HMODULE)handle)))
 #define GetDriverProc(handle, name) GetProcAddress(((HMODULE)handle), (name))
 
@@ -35,8 +45,28 @@
 #define UnloadJackModule(handle) FreeLibrary((handle));
 #define GetJackProc(handle, name) GetProcAddress((handle), (name));
 
+#ifndef ENOBUFS
 #define ENOBUFS 55
+#endif
+
+#ifdef _DEBUG
+#define JACK_DEBUG true
+#else
 #define JACK_DEBUG false
+#endif
+
+inline int setenv(const char* name, const char* value, int overwrite)
+{
+	if (overwrite == 0 && getenv(name) != NULL) {
+		return 0;
+	}
+	return _putenv_s(name, value);
+}
+
+inline int unsetenv(const char* name)
+{
+	return _putenv_s(name, "");
+}
 
 #endif
 
